
%include { 
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include "data.h" 
#include "panic.h" 
#include "mod.h" 
}

%extra_argument { struct parse_data *pd }
%token_type { struct tok_data }

%type action { struct { enum action_type atype; char *adata; } }
%type sdecls { int }

file ::= decls.

decls ::= decl decls.
decls ::= .

decl ::= KEY KEYVAL(K) action(A). {
        int end = RB_LEN(&pd->rb) / sizeof(struct map);
        struct map map;
        char *keyc = stringize_token(K);
        struct key key;
        memset(&map, 0, sizeof(map));
        key.name = strdup(keyc);
        pd->errorcount += parse_key(keyc, &key);
	free(keyc);
        map.adata = A.adata;
        map.atype = A.atype;
        map.string = "";
	map.leader = key;
        rb_append(&pd->rb, &map, sizeof(map));
        if(verbose_mode) {
                fprintf(stderr, "Appending map: ");
                dump_maps(stderr, &map, 1);
        }
}

decl ::= KEY KEYVAL(K) LBRACE sdecls(N) RBRACE. {
        int end = RB_LEN(&pd->rb) / sizeof(struct map);
        int start = end - N;
        char *keyc = stringize_token(K);
        struct key key;
        key.name = strdup(keyc);
        pd->errorcount += parse_key(keyc, &key);
        free(keyc);
        assert(pd->rb_start % sizeof(struct map) == 0);
        assert(RB_LEN(&pd->rb) % sizeof(struct map) == 0);
        print_verbose("Setting leader: %s on %i maps", key.name, end - start);
        for(;start < end; start++) {
                ((struct map *)pd->rb.buf)[start].leader = key;
        }
}

action(O) ::= EXEC(X).   { O.atype = ACTION_EXEC; O.adata = stringize_token(X); }
action(O) ::= INJECT STRINGVAL(X).    { O.atype = ACTION_INJECT; O.adata = stringize_token(X); }
action(O) ::= EXIT.   { O.atype = ACTION_EXIT; O.adata = NULL; }
action(O) ::= DIE.    { O.atype = ACTION_DIE; O.adata = NULL; }


sdecls(R) ::= sdecl sdecls(X).   { R = X + 1; }
sdecls(R) ::= .               { R = 0; pd->rb_start = RB_LEN(&pd->rb); }
sdecl ::= STRING STRINGVAL(X) action(A).  { 
      struct map map;
      memset(&map, 0, sizeof(map));
      map.adata = A.adata;
      map.atype = A.atype;
      map.string = stringize_token(X);
      
      rb_append(&pd->rb, &map, sizeof(map));
      if(verbose_mode) {
            fprintf(stderr, "Appending map: ");
            dump_maps(stderr, &map, 1);
        }
}


%syntax_error {
   fprintf(stderr, "%s:%i:syntax error\n", pd->filename, pd->line);
   pd->errorcount++;
}

%parse_failure {
   ParseFree(yypParser, free);
   longjmp(pd->error_jmp, 1);
}
