
#include <malloc.h>
#include <assert.h>
#include <string.h>
#include <stdbool.h>

#include "resizeable_buf.h"


static void
rb_makesize(rb_t *rb, size_t sz, bool preserve)
{
	assert(rb->len <= rb->size);
	assert((rb->buf && rb->size) || (!rb->buf && !rb->size));
	if(sz > rb->size) {
		if(!rb->size)
			rb->size = 2;
		while (sz > rb->size) rb->size*=2; 
		if(preserve)
			rb->buf = realloc(rb->buf, rb->size);
		else {
			free(rb->buf);
			rb->buf = malloc(rb->size);
		}
	}
	assert(rb->len <= rb->size);
	assert((rb->buf && rb->size) || (!rb->buf && !rb->size));
}

void
rb_append(rb_t *rb, void *data, size_t len)
{
	rb_makesize(rb, len + rb->len, true);
	memcpy((char *)rb->buf + rb->len, data, len);
	rb->len += len;
	assert(rb->len <= rb->size);
	assert((rb->buf && rb->size) || (!rb->buf && !rb->size));

}

void
rb_set(rb_t *rb, void *data, size_t len)
{
	rb_makesize(rb, len, false);
	memcpy(rb->buf, data, len);
	rb->len = len;
	assert(rb->len <= rb->size);
	assert((rb->buf && rb->size) || (!rb->buf && !rb->size));
}

void
rbs_set(rb_t *rb, char *str)
{
	rb_set(rb, str, strlen(str));
}

void
rbs_append(rb_t *rb, char *str)
{
	rb_append(rb, str, strlen(str));
}

void
rb_extract(rb_t *rb, rb_t *rb2, unsigned loc, size_t len)
{
	rb_set(rb, (char *)RB_PTR(rb2) + loc, len);
}


void
rb_insert(rb_t *rb, unsigned loc, char *data, size_t len)
{
	assert(rb->len <= rb->size);
	assert((rb->buf && rb->size) || (!rb->buf && !rb->size));
	assert(loc <= rb->len);
	rb_makesize(rb, rb->len + len, true);
	memmove((char *)rb->buf + loc + len, (char *)rb->buf + loc , rb->len - loc);
	memcpy((char *)rb->buf + loc, data, len);
	rb->len += len;
	
	assert(rb->len <= rb->size);
	assert((rb->buf && rb->size) || (!rb->buf && !rb->size));

}

void
rb_delete(rb_t *rb, unsigned loc, size_t len)
{
	assert(loc <= rb->len);
	assert(rb->len <= rb->size);
	assert((rb->buf && rb->size) || (!rb->buf && !rb->size));
	memmove((char *)rb->buf + loc, (char *)rb->buf + loc + len, rb->len - (loc + len));
	rb->len -= len;
	assert(rb->len <= rb->size);
	assert((rb->buf && rb->size) || (!rb->buf && !rb->size));
}

void
rb_stringize(rb_t *rb)
{
	rb_makesize(rb, rb->len + 1, true);
	((char *)rb->buf)[rb->len] = '\0';
}
